@@ -1,5 +1,5 @@
 /* ldctor.c -- constructor support routines
-   Copyright (C) 1991-2018 Free Software Foundation, Inc.
+   Copyright (C) 1991-2021 Free Software Foundation, Inc.
    By Steve Chamberlain <sac@cygnus.com>
 
    This file is part of the GNU Binutils.
@@ -23,6 +23,7 @@
 #include "bfd.h"
 #include "bfdlink.h"
 #include "safe-ctype.h"
+#include "ctf-api.h"
 
 #include "ld.h"
 #include "ldexp.h"
@@ -105,12 +106,12 @@ ldctor_add_set_entry (struct bfd_link_hash_entry *h,
     }
 
   e = (struct set_element *) xmalloc (sizeof (struct set_element));
-  e->next = NULL;
+  e->u.next = NULL;
   e->name = name;
   e->section = section;
   e->value = value;
 
-  for (epp = &p->elements; *epp != NULL; epp = &(*epp)->next)
+  for (epp = &p->elements; *epp != NULL; epp = &(*epp)->u.next)
     ;
   *epp = e;
 
@@ -151,19 +152,17 @@ ctor_prio (const char *name)
 static int
 ctor_cmp (const void *p1, const void *p2)
 {
-  const struct set_element *const *pe1
-    = (const struct set_element *const *) p1;
-  const struct set_element *const *pe2
-    = (const struct set_element *const *) p2;
+  const struct set_element *pe1 = *(const struct set_element **) p1;
+  const struct set_element *pe2 = *(const struct set_element **) p2;
   const char *n1;
   const char *n2;
   int prio1;
   int prio2;
 
-  n1 = (*pe1)->name;
+  n1 = pe1->name;
   if (n1 == NULL)
     n1 = "";
-  n2 = (*pe2)->name;
+  n2 = pe2->name;
   if (n2 == NULL)
     n2 = "";
 
@@ -177,17 +176,15 @@ ctor_cmp (const void *p1, const void *p2)
   /* We sort in reverse order because that is what g++ expects.  */
   if (prio1 < prio2)
     return 1;
-  else if (prio1 > prio2)
+  if (prio1 > prio2)
     return -1;
 
   /* Force a stable sort.  */
-
-  if (pe1 < pe2)
+  if (pe1->u.idx < pe2->u.idx)
     return -1;
-  else if (pe1 > pe2)
+  if (pe1->u.idx > pe2->u.idx)
     return 1;
-  else
-    return 0;
+  return 0;
 }
 
 /* This function is called after the first phase of the link and
@@ -213,22 +210,24 @@ ldctor_build_sets (void)
       for (p = sets; p != NULL; p = p->next)
 	{
 	  int c, i;
-	  struct set_element *e;
+	  struct set_element *e, *enext;
 	  struct set_element **array;
 
 	  if (p->elements == NULL)
 	    continue;
 
 	  c = 0;
-	  for (e = p->elements; e != NULL; e = e->next)
+	  for (e = p->elements; e != NULL; e = e->u.next)
 	    ++c;
 
 	  array = (struct set_element **) xmalloc (c * sizeof *array);
 
 	  i = 0;
-	  for (e = p->elements; e != NULL; e = e->next)
+	  for (e = p->elements; e != NULL; e = enext)
 	    {
 	      array[i] = e;
+	      enext = e->u.next;
+	      e->u.idx = i;
 	      ++i;
 	    }
 
@@ -237,8 +236,8 @@ ldctor_build_sets (void)
 	  e = array[0];
 	  p->elements = e;
 	  for (i = 0; i < c - 1; i++)
-	    array[i]->next = array[i + 1];
-	  array[i]->next = NULL;
+	    array[i]->u.next = array[i + 1];
+	  array[i]->u.next = NULL;
 
 	  free (array);
 	}
@@ -293,7 +292,7 @@ ldctor_build_sets (void)
 	      /* See PR 20911 for a reproducer.  */
 	      if (p->elements->section->owner == NULL)
 		einfo (_("%X%P: special section %s does not support reloc %s for set %s\n"),
-		       bfd_get_section_name (link_info.output_bfd, p->elements->section),
+		       bfd_section_name (p->elements->section),
 		       bfd_get_reloc_code_name (p->reloc),
 		       p->h->root.string);
 	      else
@@ -333,7 +332,7 @@ ldctor_build_sets (void)
 				       FALSE));
       lang_add_data (size, exp_intop (p->count));
 
-      for (e = p->elements; e != NULL; e = e->next)
+      for (e = p->elements; e != NULL; e = e->u.next)
 	{
 	  if (config.map_file != NULL)
 	    {
